DiscoPoP is an open-source tool that helps software developers parallelize their programs with threads. It is a joint project of Technical University of Darmstadt and Iowa State University.

DiscoPoP is built on top of LLVM. 

## Getting started
### Pre-requisites
Before doing anything, you'll need a basic development setup. We have tested DiscoPoP on Unbuntu and following packages should be installed before installing the profiler:

	sudo apt-get install git build-essential cmake

Additionally, you need LLVM installed on your system. Currently, DiscoPoP supports LLVM 8.0.1. Lower versions are not supported, due to API changes which lead to compilation failure. Please follow the installation tutorial [here](https://llvm.org/docs/GettingStarted.html), if you have not installed LLVM before.

***** LLVM INSTALLATION PROBLEM? *****

Please skip this section if you do not have problems installing llvm on your machine.

If you have problems installing LLVM on your system, We have provided other options to evaluate the artifact:

1. Docker image

   We have created a docker image that you can simply log in and run the test programs. We have put the docker image in a dropbox folder. Below, you can find the link to the folder

   **** NICK: PLEASE PASTE THE LINK TO THE DROPBOX FOLDER HERE ****

2. Access to our test system

   In case you cannot use the docker image, please log in to our test system. Below you can find the access information:

   ssh europar2020@130.83.143.241
   Password: r5K)Cexf.G`/g#VK

   You can find the installed LLVM, DiscoPoP and our approach in /media/backuphd/europar2020/ 

*************************************

Below you can find the instructions about how to install and use DiscoPoP and our approach to profile programs.

The vanilla version of the data-dependnece profiler of DiscoPoP is in "DPInstrumentation" directory. The source code for our approach which is proposed in the paper is in "DPInstrumentationOmission". 

Below, you can find the instructions for the installation of the DiscoPoP profiler in addition to the installation of our approach.

### Installation
First, create a build directory:

	mkdir build; cd build;

Next configure cmake. The preferred LLVM installation path for DiscoPoP can be set using the -DLLVM_DIST_PATH=<PATH_TO_LLVM_BUILD_FOLDER> cmake variable.

	cmake -DLLVM_DIST_PATH=<PATH_TO_LLVM_BUILD_FOLDER> ..

Once the configuration process is successfully finished, run `make` to compile the profiler to obtain the profiling libraries.


### Running DiscoPoP
In the following, we will explain how to run the data-dependence profiler of DiscoPoP and our approach. However, before the execution, run the `dp-fmap` script in the root folder of the target application to obtain the list of files. The output will be written in a file named `FileMapping.txt`.

	<DISCOPOP_PATH>/scripts/dp-fmap

Please use the exact compiler flags that we used. Otherwise, you might not get the correct results, or the analysis might fail.

#### DiscoPoP Profiling
To obtain data dependences, we need to instrument the target application: 

	clang -g -O0 -S -emit-llvm -fno-discard-value-names -Xclang -load -Xclang <PATH_TO_DISCOPOP_BUILD_DIR>/libi/LLVMDPInstrumentation.so -mllvm -fm-path -mllvm ./FileMapping.txt -c <C_File> -o out.ll

(OPTIONAL) If your test project has multiple files, please use llvm-link to link multiple .ll files into one:

	llvm-link $out.ll other.ll -o out.ll

#### Eliminating non-essential instructions from profiling (OUR APPROACH)
You may run the DPInstrumentationOmission pass to omit non-essential instructions from profiling to improve the performance without harming the accuracy of the extracted data dependences:

	opt -S -load=<PATH_TO_DISCOPOP_BUILD_DIR>/libi/LLVMDPInstrumentationOmission.so -dp-instrumentation-omission -stats out.ll -o out.ll

Make the application executable:
	
	clang++ out.ll-L<PATH_TO_DISCOPOP_BUILD_DIR>/rtlib -lDiscoPoP_RT -lpthread -o <APP_NAME>

 Running the application will result in a text file containing all the dependences located in the present working directory

	./<APP_NAME>

To observe the reduction in the profiling time, please profile the program with and without our approach.

### Walk-through example

The following walk-through example demonstrates how to use DiscoPoP and our approach to analyze a sequential sample application. In this example, we use the program fib. We assume that you have successfully installed DiscoPoP and our approach. We have created a build directory which contains the installation files of DiscoPoP and our approach.

First, switch to the test/fibonacci folder which contains the program fib.c. Then, please run the following commands step-by-step to obtain the desired results.

    Run the dp-fmap script to obtain the list of files. The output will be written in a file named FileMapping.txt.

    To obtain data dependences, we need to instrument the application and run it.

    clang -g -O0 -fno-discard-value-names -emit-llvm -Xclang -load -Xclang ../../build/libi/LLVMDPInstrumentation.so -mllvm -fm-path -mllvm ./FileMapping.txt -c fib.c -o fib-discopop.ll
    
    clang++ fib-discopop.ll -L../../build/rtlib -lDiscoPoP_RT -lpthread -o fib-discopop

    With the following command, we run the program to obtain data dependences with DiscoPoP profiler. We used time command in linux to get the profiling time. Please feel free to use any other time measurement commands/tools etc.

    time ./fib-discopop

The output is a text file that contains all the dependences. For the example, you can find the data dependences in test/fibonacci/fib-discopop_dep.txt. A data dependence is represented as a triple <sink, type, source>. type is the dependence type (RAW, WAR or WAW). Note that a special type INIT represents the first write operation to a memory address. source and sink are the source code locations of the former and the latter memory access, respectively. sink is further represented as a pair <fileID:lineID>, while source is represented as a triple <fileID:lineID|variableName>. The keyword NOM (short for "NORMAL") indicates that the source line specified by aggregated sink has no control-flow information. Otherwise, BGN and END represent the entry and exit points of a control region, respectively.

Further, we use our approach to remove the instrumentation of memory-access instructions whose data dependences can be identified statically by our approach. Again, we need to execute the instrumentation step on the fib program. Please note that we could reuse the instrumented file in the previous step. However, we repeated it for the sake of clarity.

	clang -g -O0 -fno-discard-value-names -emit-llvm -Xclang -load -Xclang ../../build/libi/LLVMDPInstrumentation.so -mllvm -fm-path -mllvm ./FileMapping.txt -c fib.c -o fib-our-approach.ll

	opt -S -load=../../build/libi/LLVMDPInstrumentationOmission.so -dp-instrumentation-omission -stats fib-our-approach.ll -o fib-our-approach.ll

	clang++ fib-our-approach.ll -L../../build/rtlib -lDiscoPoP_RT -lpthread -o fib-our-approach

	time ./fib-our-approach

The output is again a txt file (i.e., ./test/fibonacci/fib-our-approach_dep.txt) which contains data dependences extracted by our approach.

Please compare the profiling time with and without our approach (i.e., the execution time of "fib-discopop" and the execution time of "fib-our-approach"). They are significantly different (around 80% reduction in the profiling time).

Also, please compare the data depedences identified by both tools. Our approach identifies two more data dependences: 

> 1:19 NOM  RAW 1:16|n RAW 1:18|res

Both of them are transitive data dependences as explained in the paper.

## License
© Contributors Licensed under an BSD-3-Clause license.
