#  EuroPar 2020
## [105] Skipping Non-essential Instructions Makes Data-dependence Profiling Faster
### Artifact Testing Manual

This document describes how to run the tests which provided the results of our paper. Three benchmark suites were used: [BOTS](https://github.com/bsc-pm/bots), [Polybench](http://web.cs.ucla.edu/~pouchet/software/polybench/) and [NPB](https://www.nas.nasa.gov/publications/npb.html). 

The process was similar for all three, with some small differences in the steps. The instructions below are for running our tests for all of the programs in each benchmark suite.

## Prepare Environments (Can be skipped if using Docker Image)

### Prepare Env Variables
Store the path to both the DiscoPoP and the LLVM build directories in environment variables
    
    export LLVM_BUILD=<PATH_TO_YOUR_LLVM_BUILD>
    export DP_BUILD=<PATH_TO_YOUR_DP_BUILD>

### BOTS
	cd bots
    <PATH_TO_CLONED_DP_REPO>/scripts/dp-fmap .
    ./configure
    ## Select '3. dp'
    cd serial
    make
    cd ..
    
### Polybench
	cd Poly
    <PATH_TO_CLONED_DP_REPO>/scripts/dp-fmap .
    ./compile-all.sh
    # Select 2) dp

### NAS
    cd NAS
    <PATH_TO_CLONED_DP_REPO>/scripts/dp-fmap .
    ./make-all.sh <CLASS>

Where \<CLASS> is one of the following:
* S: small for quick test purposes
* W: workstation size (a 90's workstation; now likely too small)
* A, B, C: standard test problems; ~4X size increase going from one class to the next
* D, E, F: large test problems; ~16X size increase from each of the previous classes


## Run Profilers
For all 3 benchmarks the process to run the DiscoPoP Profiler with and without our omissions is the same and is described in the steps below.
**Note**: Please cd into the respective benchmark directory before running the commands below.

##### Profile all programs with vanilla DiscoPoP Profiler
    ./make-all-runnable.sh
    ## Select 2) dp

##### Run all programs profiled with vanilla DiscoPoP
    ./run-all.sh <N> DEPS_VANILLA
Replace \<N> with the desired number of runs per program. The identified dependences will be in the `bots/DEPS_VANILLA` folder and the performance results will be under `bots/DEPS_VANILLA/run-all.log`

##### Profile all programs with the DiscoPoP Profiler enhanced with static omissions
    ./make-all-runnable.sh
    ## Select 3) dp+SPA

##### Run all programs profiled with the DiscoPoP Profiler enhanced with static omissions
    ./run-all.sh <N> DEPS_OMISSIONS
Replace \<N> with the desired number of runs per program. The identified dependences will be in the `bots/DEPS_OMISSIONS` folder and the performance results will be under `bots/DEPS_OMISSIONS/run-all.log`

**NOTE**: You can modify the list of programs which are run directly in the `run-all.sh` script in each benchmark directory

**NOTE**: You can run a single target program \<N> times by runnning:
`./run-n-times <N> run_custom/run-<TARGET>.sh`

## Compare dependences identified by each method
You can use our comparison script to facilitate the comparison of dependence files generated by both methods (with and without omissions). In each of the benchmark folders, after following the instructions to profile the programs with both methods, run the following script:

    ./compare-all-deps.sh DEPS_VANILLA DEPS_OMISSIONS
 
 The script will compare all of the dependences identified for each program in each of the folders DEPS_VANILLA and DEPS_OMISSIONS. It can identify most transitive dependences and will not print these. Discrepancies that it could not identify as transitive dependences are reported under `Possibly Non-Transitive Deps`. Manually investigating each dependence proves to be either an improvement in accuracy (elimination of false-negatives/positives) or a transitive dependence not identifiable by our filtering algorithm.