import re
import sys

class Dep:
    def __init__(self, line, var):
        self.line = line
        self.var = var
class Line:
    def __init__(self, line):
        self.line = line
        self.out_deps = []
    def add_out_dep(self, line, var):
        for d in self.out_deps:
            if d.line == line and d.var == var:
                return
        self.out_deps.append(Dep(line, var))
class DepGraph:
    def __init__(self):
        self.line_set = []

    def add_line(self, line):
        for l in self.line_set:
            if l.line == line:
                return l
        l = Line(line)
        self.line_set.append(l)
        return l

    def get_line(self, line):
        for l in self.line_set:
            if l.line == line:
                return l

    def dfs(self, current, searched, var, visited=None):
        if current is None or searched is None:
            return False
        if current.line == searched.line:
            return True
        if visited is None:
            visited = set()
        visited.add(current.line)
        for dep in current.out_deps:
            next = self.get_line(dep.line)
            if dep.var == var and next.line not in visited:
                if self.dfs(next, searched, var, visited):
                    return True
        return False
    
    def hasPathOverVar(self, srcLine, dstLine, var):
        return self.dfs(self.get_line(srcLine), self.get_line(dstLine), var)
        

depRegex = r"[ARW]+ [0-9]+:[0-9]+\|\S*"
lineRegex = r"[0-9]+:[0-9]+"

def read_deps(path):
    print("Getting deps from " + path)
    dep_set = set()
    dg = DepGraph()
    fp = open(path, "r")
    lines = fp.readlines() 
    for line in lines: 
        src = re.findall(lineRegex, line)[0]
        srcLine = dg.add_line(src)
        lineDeps = re.findall(depRegex, line)
        for dep in lineDeps:
            dep_set.add(src + " " + dep)
            
            dst = re.findall(lineRegex, dep)[0]
            dstVar = dep[dep.find('|')+1:len(dep)]

            srcLine.add_out_dep(dst, dstVar)
            
            dstLine = dg.add_line(dst)
    fp.close()
    return {'s': dep_set, 'dg': dg}

def get_transitive_deps_in_set(D):
    transitive_dep_count = 0
    possibly_non_transitive_deps=""
    for dep in D2:
        lines = re.findall(lineRegex, dep)
        src = lines[0]
        dst = lines[1]
        var = dep[dep.find('|')+1:len(dep)]
        if dg1.hasPathOverVar(src, dst, var):
            transitive_dep_count = transitive_dep_count + 1
        else:
            possibly_non_transitive_deps += "\n\t\t{}".format(dep)
    print("\tTransitive Deps: {}".format(transitive_dep_count))
    print("\tPossibly Non-Transitive Deps: {}".format(len(D2) - transitive_dep_count))
    print(possibly_non_transitive_deps)
r1 = read_deps(sys.argv[1])
r2 = read_deps(sys.argv[2])
S1 = r1['s']
S2 = r2['s']
dg1 = r1['dg']
dg2 = r2['dg']


print("Dependencies in S1 ({}): {}".format(sys.argv[1], len(S1)))
print("Dependencies in S2 ({}): {}".format(sys.argv[2], len(S2)))

D1 = S1.difference(S2)
D2 = S2.difference(S1)

print("|S2-S1| (Additional): {}".format(len(D2)))
get_transitive_deps_in_set(D2)

print("|S1-S2| (Missing): {}".format(len(D1)))
get_transitive_deps_in_set(D1)